/* 
This example is aimed to show the overall scheme typically used to measure 
the runtime of a portion of code. Here, the target snippet is represented by the 
method "foo". In the case of Java code, getting meaningful figures is quite 
complicated, because of the behaviour of the supporting platform.
Thus, in a real setting, this overall scheme has to be significantly improved by 
making use of specific tricks.
*/

package cds;

import java.util.concurrent.atomic.AtomicInteger;


public class CDS_04_TimerTest {
    public static int NUMWARMUP=15000;//number of warmup iterations
    public static int NUMTESTS=10;    //number of tests to be performed
    public static int NUMITERS=100000; //number of times to repeat the execution
    
    public static AtomicInteger h=new AtomicInteger(4);
 
    /*
    operations in method Foo are meaningles...
    */
    public static long foo(int x){
        
        h.compareAndSet(x%10, (x%10)+1);
        long y= x*h.get();
        y = 0xFFF & y;
        //*
        for (int i=2; i<400; i++) {
            ++y;
            y+=(i%59);
            y*=y;
            y/=(x%2+1);
            y%=73;
        }
                //*/
        return y;
    }
     
    
    public static void main(String[] args){
        
        long res;
        
        warmUpFoo();
        System.out.println("");
        
        measureDeadCodeFoo();
        System.out.println("");
        
        res=measureNoDeadCodeFoo();
        System.out.println("");
        
        System.out.print(res);
        
        
    }  
 
    

    public static void warmUpFoo(){
        for(int i=0; i<NUMWARMUP;i++){
            
            foo(3); //the computed results are not used any more out of this loop!
                          
            
        }
        System.out.printf("Warmup over\n");
    }   
    
    public static void measureDeadCodeFoo(){
        System.out.println("\nTESTS with possible dead code detection");
        
        for(int i=0; i<NUMTESTS;i++){
            
            MyTimer.clear();
            MyTimer.start();
            
            for(int j=1; j<=NUMITERS;j++){
                foo(j); //the computed results are not used any more out of this loop!
            }
            MyTimer.stop();
                          
            System.out.printf("Test #%3d:\t%6.1f ns \t~%4.0f CPU cycles%n",i+1,(float)MyTimer.read()/NUMITERS, (float)MyTimer.readCycles()/NUMITERS);
        }
        
    }
    
    public static long measureNoDeadCodeFoo(){

        long dummyres=0, d = 0;
        System.out.println("\nTESTS with trick to avoid dead code detection\n");
        
        for(int i=0; i<NUMTESTS;i++){
            
            MyTimer.clear();
            MyTimer.start();
                        
            for(int j=1; j<=NUMITERS;j++){
                dummyres+=foo(j)%10;
            }
            MyTimer.stop();
               
            System.out.printf("Test #%3d:\t%6.1f ns \t~%4.0f cycles \t res=%d%n",i+1,(float)MyTimer.read()/NUMITERS, (float)MyTimer.readCycles()/NUMITERS, dummyres);
        }

        return dummyres;
        
    }
    
    
    
}
